import express from 'express'
import cors from 'cors'
import bodyParser from 'body-parser'
import mongoose from 'mongoose'
import configFile from './config/database'

import users from './routes/users'
import swisstennis from './routes/swisstennis'
import sztm from './routes/sztm'
import config from './routes/config'
import download from './routes/download'
import sms from './routes/sms'

import { authenticate, verify } from './routes/authenticate'

import { errorHandler } from './middleware/apiErrorHandler'

mongoose.connect(configFile.database)

const port = process.env.PORT || 3002
const app = express()
app.use(cors())

app.use(bodyParser.urlencoded({ extended: false }))
app.use(bodyParser.json())

app.get('/', (req, res) => {
  res.send(`Express API`)
})

app.use('/authenticate', authenticate)
app.use('/download', download)

const apiRoutes = express.Router()
apiRoutes.use('/users', users)
apiRoutes.use('/swisstennis', swisstennis)
apiRoutes.use('/sztm', sztm)
apiRoutes.use('/config', config)
apiRoutes.use('/sms', sms)
app.use('/api', verify)
app.use('/api', apiRoutes)
app.use(errorHandler)

app.listen(port)
console.log(`Server running on port ${port}.`)