import express from 'express'
import { wrapAsync } from '../middleware/apiErrorHandler'
import Config from '../models/config'

const config = express.Router()

config.post('/', wrapAsync(async (req, res) => {
    try{
        const { key, value, description } = req.body
        if (!key || !value) {
            throw Error('parameters key and value are mandatory.')
        }
        const configPair = new Config({ key, value, description })
        await configPair.save()
        res.json(configPair)
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }
}))

config.get('/', wrapAsync(async (req, res) => {
    try {
        const config = await Config.find()
        res.json({ config })
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    } 
}))

config.get('/:key', wrapAsync(async (req, res) => {
    try {
        const { key } = req.params
        if (!key) {
            throw Error('parameter key is mandatory.')
        }
        const config = await Config.findOne({ key })
        res.json({ config })
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }  
}))

config.put('/', wrapAsync(async (req, res) => {
    try{
        const { key, value, description } = req.body
        if (!key || !value) {
            throw Error('parameters key and value are mandatory.')
        }
        const updated = await Config.findOneAndUpdate({ key }, { value, description })
        if (!updated) {
            throw Error('key not found!')
        }  
        const config = await Config.findOne({ key })   
        res.json(config)   
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }
}))

config.delete('/', wrapAsync(async (req, res) => {
    try{
        const { key } = req.body
        if (!key) {
            throw Error('parameter key is mandatory.')
        }
        const deleted = await Config.findOneAndRemove({ key })
        if (deleted) {
            res.json({ key })
        } else {
            throw Error('key not found!') 
        }
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }
}))

export default config