import fs from 'fs'
import express from 'express'
import moment from 'moment'
import { places } from '../config/sztm'
import Match from '../models/match'
import Player from '../models/player'
import MatchList from '../models/matchList'
import PlayerList from '../models/playerList'
import pdfMake from "pdfmake/build/pdfmake";
import pdfFonts from "pdfmake/build/vfs_fonts";
pdfMake.vfs = pdfFonts.pdfMake.vfs;

const sztm = express.Router() 

sztm.get('/schedule', async (req, res) => {
    try{
        console.log(req.query)
        const { matchListId, date, place, category } = req.query
        
        const matchList = matchListId ? 
            await MatchList.findOne({ _id: matchListId }) : 
            await MatchList.findOne().sort({ imported: -1 })
        
        console.log('matches', matchList.matches.length)
        const query = { _id: { $in: matchList.matches } }
        /*if (place) query.place = place
        if (category) query.category = category
        if (date) { 
            query.date = {} 
            const dateElems = date.match(/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/).slice(1, 7)
            const startDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 00:00:00`)
            query.date.$gte = startDate
            const endDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 23:59:59`)
            query.date.$lte = endDate
        }*/

        const matches = await Match.find(query).sort({ date: 1 }).populate('player1').populate('player2')
        console.log('matches', matches.length)
        //const players = await Player.find({ _id: { $in: playerList } })
        //console.log(playerList, players)
        res.json({ matches })
        
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }
})

sztm.get('/players', async (req, res) => {
    try{
        console.log(req.query)
        const { playerListId, date, place, category } = req.query
        
        const playerList = playerListId ? 
            await PlayerList.findOne({ _id: playerListId }) : 
            await PlayerList.findOne().sort({ imported: -1 })
        
        console.log('players', playerList.players.length)
        const query = { _id: { $in: playerList.players } }
        /*if (place) query.place = place
        if (category) query.category = category
        if (date) { 
            query.date = {} 
            const dateElems = date.match(/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/).slice(1, 7)
            const startDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 00:00:00`)
            query.date.$gte = startDate
            const endDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 23:59:59`)
            query.date.$lte = endDate
        }*/

        const players = await Player.find(query).sort({ date: 1 })
        console.log('players', players.length)
        
        res.json({ players })
        
    } catch (error) {
        res.status(400).json({ msg: error.toString() })
    }
})

sztm.post('/pdf', async (req, res) => {
    try {
        const { matchListId, date, place, category } = req.body
        
        const matchList = matchListId ? 
            await MatchList.findOne({ _id: matchListId }) : 
            await MatchList.findOne().sort({ imported: -1 })
        console.log(matchList._id)
        
        const query = { _id: { $in: matchList.matches } }
        if (place) query.place = place
        if (category) query.category = category
        if (date) { 
            query.date = {} 
            const dateElems = date.match(/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/).slice(1, 7)
            const startDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 00:00:00`)
            query.date.$gte = startDate
            const endDate = new Date(`${dateElems[0]}-${dateElems[1]}-${dateElems[2]} 23:59:59`)
            query.date.$lte = endDate
        }
        
        const matches = await Match.find(query).sort({ date: 1 }).populate('player1').populate('player2')
        const players = []
        matches.forEach(match => {
            if (match.player1) players.push(match.player1)
            if (match.player2) players.push(match.player2)
        })

        const allPlaces = []
        matches.forEach(match => {
            if (!!match.place && !allPlaces.includes(match.place)) allPlaces.push(match.place)
        })

        allPlaces.forEach(place => {
            const tableRows = []
            matches.filter(match => match.place && (match.place == place)).forEach(match => {
                if (match.player1) tableRows.push([match.player1.paid ? 'OK' : '', match.category, moment(match.date).format('HH:mm'), `(${match.player1.junior ? '30.-' : '50.-'}) ${match.player1.fullName}`, ''])
                if (match.player2) tableRows.push([match.player2.paid ? 'OK' : '', match.category, moment(match.date).format('HH:mm'), `(${match.player2.junior ? '30.-' : '50.-'}) ${match.player2.fullName}`, ''])
            })
            const paylistDoc = {
                pageSize: 'A4',
                pageOrientation: 'portrait',
                content: [
                    { text: 'Stadtzürcher Tennismeisterschaft', style: 'header' },
                    { text: `Nenngelder für den ${moment(query.date.$gte).format('DD.MM.YYYY')}`, style: 'subheader' },
                    { columns: [
                        {text: places[place]},
                        {text: '50.- oder 30.- (Junioren Jg. 2000 oder jünger)'}
                    ]},
                    { table: {
                        headerRows: 1,
                        widths: ['10%', '20%', '10%', '*', '15%'],
                        heights: 30,
                        body: [
                            [{text: 'bezahlt', fillColor: '#eeeeee'}, 
                             {text: 'Kategorie', fillColor: '#eeeeee'}, 
                             {text: 'Zeit', fillColor: '#eeeeee'}, 
                             {text: 'Name', fillColor: '#eeeeee'}, 
                             {text: 'Betrag bez.', fillColor: '#eeeeee'}],
                            ...tableRows
                        ]},
                        style: 'tableStyle'
                    },
                    { table: {
                        dontBreakRows: true,
                        widths: ['25%', '*', '25%', '25%'],
                        body: [
                            [{text: 'Datum', bold: true}, {text: '', colSpan: 3}, {}, {}],
                            [{text: 'Turnierleiter', colSpan:2, margin: [0, 10, 0, 0], bold: true}, {}, 
                             {text: 'Kassierer', colSpan:2, margin: [0, 10, 0, 0], bold: true}, {}],
                            [{text: 'Betrag von Spielern erhalten', colSpan:2, margin: [0, 25, 0, 0]}, {}, 
                             {text: 'Betrag von Turnierleiter erhalten', colSpan:2, margin: [0, 25, 0, 0]}, {}]
                        ]},
                      style: 'tableStyle'}
                    ],
                styles: {
                    header: {
                        fontSize: 22,
                        bold: true,
                        margin: [0, 20, 0, 8]
                    },
                    subheader: {
                        fontSize: 14,
                        margin: [0, 15, 0, 12]
                    },
                    tableStyle: {
                        margin: [0, 15, 0, 5]
                    }
                }
            }
    
            const scheduleDoc = {
                pageSize: 'A4',
                pageOrientation: 'landscape',
                content: [
                    { text: 'Stadtzürcher Tennismeisterschaft', style: 'header' },
                    { text: `Spielplan für den ${moment(query.date.$gte).format('DD.MM.YYYY')} (${places[place]})`, style: 'subheader' },
                    { table: {
                        headerRows: 1,
                        widths: ['5%', '5%', '12%', '*', '5%', '*', '5%', '7%', '7%', '7%', '10%'],
                        heights: 30,
                        body: [
                            [{text: 'Platz', fillColor: '#eeeeee'}, 
                             {text: 'Zeit', fillColor: '#eeeeee'}, 
                             {text: 'Kategorie', fillColor: '#eeeeee'}, 
                             {text: 'Spieler 1', fillColor: '#eeeeee', colSpan: 2}, 
                             {}, 
                             {text: 'Spieler 2', fillColor: '#eeeeee', colSpan: 2}, 
                             {}, 
                             {text: '1. Satz', fillColor: '#eeeeee'}, 
                             {text: '2. Satz', fillColor: '#eeeeee'}, 
                             {text: '3. Satz', fillColor: '#eeeeee'}, 
                             {text: 'WO Grund', fillColor: '#eeeeee'}],
                            ...matches.filter(match => match.place && (match.place == place)).map((match) => {
                                return ['', moment(match.date).format('HH:mm'), match.category, 
                                match.player1 && match.player1.fullName, match.player1 && match.player1.ranking, 
                                match.player2 && match.player2.fullName, match.player2 && match.player2.ranking, 
                                '', '', '', '']
                            })
                        ]},
                        style: 'tableStyle'
                    }],
                styles: {
                    header: {
                        fontSize: 22,
                        bold: true,
                        margin: [0, 20, 0, 8]
                    },
                    subheader: {
                        fontSize: 14,
                        margin: [0, 15, 0, 12]
                    },
                    tableStyle: {
                        margin: [0, 15, 0, 5]
                    } 
                }
            }
    
            const paylistGenerator = pdfMake.createPdf(paylistDoc)
            paylistGenerator.getBuffer((buffer) => {
                fs.writeFileSync(`/usr/src/sztm_files/Zahlliste-${place}-${moment(query.date.$gte).format('YYYYMMDD')}.pdf`, new Buffer(new Uint8Array(buffer)))
            })
            const scheduleGenerator = pdfMake.createPdf(scheduleDoc)
            scheduleGenerator.getBuffer((buffer) => {
                fs.writeFileSync(`/usr/src/sztm_files/Spielliste-${place}-${moment(query.date.$gte).format('YYYYMMDD')}.pdf`, new Buffer(new Uint8Array(buffer)))
            })

        })
        
        res.json({ msg: "All PDFs generated." })
    } catch (error) {
        return res.json({ msg: error.toString(), stack: error.stack })
    }
})

export default sztm